@@ -1,10 +1,10 @@
 /*
+ * $Id: darwinusb.c,v 1.66 2006/11/09 10:26:21 fpillet Exp $
+ *
  * darwinusb.c: I/O support for Darwin (Mac OS X) USB
  *
  * Copyright (c) 2004-2006, Florent Pillet.
  *
- * $Id: darwinusb.c,v 1.61 2006/08/27 18:51:37 desrod Exp $
- *
  * libpisock interface modeled after linuxusb.c by Jeff Dionne and 
  * Kenneth Albanowski
  * Some structures & defines extracted from Linux "visor.c",
@@ -154,6 +154,7 @@ typedef struct usb_connection_t
 	int device_present;
 	int read_pending;			/* set to 1 when a prime_read() has been issued and the read_completion() has not been called yet */
 	int in_pipe_ref;			/* pipe for reads */
+	int in_pipe_bulk_size;		/* max packet size of bulk packets on input */
 	int out_pipe_ref;			/* pipe for writes */
 	int out_pipe_bulk_size;			/* size of bulk packets on the out pipe (used when talking with a PalmConnect USB serial adapter) */
 
@@ -348,7 +349,7 @@ acceptedDevices[] = {
 	{0x0830, 0x0052},
 	{0x0830, 0x0053},
 	{0x0830, 0x0060, FLAG_ANSWERS_CONN_INFO},	/* Tungsten series, Zire 71 */
-	{0x0830, 0x0061, FLAG_ANSWERS_PALM_CONN_INFO | FLAG_USE_FIRST_PAIR},	/* Zire 22, 31, 72, T|5, T|X, LifeDrive -- for T|X and LD, they don't answer to the PALM_EXT_CONNECTION_INFORMATION control code. In this case we revert to using the first pair of pipes */
+	{0x0830, 0x0061, FLAG_ANSWERS_PALM_CONN_INFO | FLAG_USE_FIRST_PAIR},	/* Zire 22, 31, 72, T|5, T|X, LifeDrive, Treo 650 -- for T|X and LD, they don't answer to the PALM_EXT_CONNECTION_INFORMATION control code. In this case we revert to using the first pair of pipes */
 	{0x0830, 0x0062},
 	{0x0830, 0x0063},
 	{0x0830, 0x0070, FLAG_ANSWERS_CONN_INFO},	/* Zire */
@@ -954,7 +955,7 @@ configure_device(IOUSBDeviceInterface **di,
 	if (kr == kIOReturnNotReady)
 		return kr;
 
-	/* With some devices (Palm) we need to hardcord the location of the pipes to use
+	/* With some devices (Palm) we need to hardcode the location of the pipes to use
 	 * because the Palm engineers had the good idea to mismatch the connection information data
 	 * and the actual pipes to use
 	 */
@@ -1154,9 +1155,10 @@ find_interfaces(usb_connection_t *c,
 						if ((pass == 1 && input_pipe_number != 0xff && number == input_pipe_number) ||
 						    (pass == 1 && port_number == 0xff && input_pipe_number == 0xff && (accept_flags & FLAG_USE_FIRST_PAIR)) ||
 						    (pass == 2 && port_number != 0xff && number == port_number) ||
-						    (pass == 3 && ((port_number != 0xff && pair_index == port_number) || (port_number == 0xff && maxPacketSize == 64))) ||
+						    (pass == 3 && ((port_number != 0xff && pair_index == port_number) || (port_number == 0xff && (maxPacketSize == 64 || maxPacketSize == 512)))) ||
 						     pass == 4)
 							c->in_pipe_ref = pipeRef;
+							c->in_pipe_bulk_size = maxPacketSize;
 					}
 					else if (c->out_pipe_ref == 0 &&
 					         direction == kUSBOut &&
@@ -1165,7 +1167,7 @@ find_interfaces(usb_connection_t *c,
 						if ((pass == 1 && output_pipe_number != 0xff && number == output_pipe_number) ||
 						    (pass == 1 && port_number == 0xff && input_pipe_number == 0xff && (accept_flags & FLAG_USE_FIRST_PAIR)) ||
 						    (pass == 2 && port_number != 0xff && number == port_number) ||
-						    (pass == 3 && ((port_number != 0xff && pair_index == port_number) || (port_number == 0xff && maxPacketSize == 64))) ||
+						    (pass == 3 && ((port_number != 0xff && pair_index == port_number) || (port_number == 0xff && (maxPacketSize == 64 || maxPacketSize == 512)))) ||
 						     pass == 4)
 						{
 							c->out_pipe_ref = pipeRef;
@@ -1294,7 +1296,7 @@ decode_generic_connection_information(palm_ext_connection_info *ci, int *port_nu
 			 */
 			return kIOReturnNotReady;
 		}
-		if (port_function_id == 'sync')
+		if (port_function_id == 'sync' || port_function_id == 'ppp_')
 		{
 			/* we found the port/pipes to use for synchronization
 			 * If endpoint_numbers_different is != 0, then the number of each
@@ -1510,13 +1512,13 @@ prime_read(usb_connection_t *c)
 		}
 		else
 		{
-			c->last_read_ahead_size = c->read_ahead_size & ~63;
+			c->last_read_ahead_size = c->read_ahead_size & ~(c->in_pipe_bulk_size-1);
 			if (c->last_read_ahead_size <= 0)
 				c->last_read_ahead_size = c->auto_read_size;
 			if (c->last_read_ahead_size > MAX_AUTO_READ_SIZE)
 				c->last_read_ahead_size = MAX_AUTO_READ_SIZE;
-			else if (c->last_read_ahead_size <= 0)
-				c->last_read_ahead_size = 64;			// USB packet size
+			else if (c->last_read_ahead_size < c->in_pipe_bulk_size)
+				c->last_read_ahead_size = c->in_pipe_bulk_size;			// USB packet size
 		}
 
 		ULOG((PI_DBG_DEV, PI_DBG_LVL_DEBUG, "darwinusb: prime_read(%p) for %d bytes\n", c, c->last_read_ahead_size));
@@ -1958,8 +1960,9 @@ pi_usb_impl_init (struct pi_usb_impl *impl)
 	impl->control_request	= NULL;   /* that is, until we factor out common code */
 }
 
-/* vi: set ts=4 sw=4 sts=4 noexpandtab: cin */
+/* vi: set ts=8 sw=4 sts=4 noexpandtab: cin */
+/* ex: set tabstop=4 expandtab: */
 /* Local Variables: */
 /* indent-tabs-mode: t */
-/* c-basic-offset: 4 */
+/* c-basic-offset: 8 */
 /* End: */
