@@ -1,5 +1,5 @@
 /*
- * $Id: usb.c,v 1.55 2007-11-09 13:59:15 desrod Exp $
+ * $Id: usb.c,v 1.57 2009/05/25 04:19:46 desrod Exp $
  *
  * usb.c: Interface layer to serial HotSync connections
  *
@@ -698,6 +698,7 @@ pi_usb_dev_t known_devices[] = {
 		.vendor 	= 0x054c,
 		.product 	= 0x0169,
 		.idstr 		= "Sony TJ",
+		.flags 		= USB_INIT_SONY_CLIE,
 	},
 
 	/* AlphaSmart */
@@ -810,7 +811,8 @@ pi_usb_dev_t known_devices[] = {
 	{
 		.vendor 	= 0x0830,
 		.product 	= 0x0061,
-		.idstr 		= "Zire 31, 72",
+		.idstr		= "Zire 31, 72, Z22",
+		.flags		= USB_INIT_TAPWAVE,
 	},
 
 	{
