@@ -1,5 +1,5 @@
 /*
- * $Id: pilot-schlep.c,v 1.46 2006/10/12 14:21:21 desrod Exp $ 
+ * $Id: pilot-schlep.c,v 1.48 2009/06/04 13:38:47 desrod Exp $ 
  *
  * pilot-schlep.c:  Utility to transfer arbitrary data to/from your Palm
  *
@@ -22,6 +22,7 @@
  */
 
 #include <stdio.h>
+#include <sys/stat.h>
 
 #include "pi-source.h"
 #include "pi-file.h"
@@ -38,7 +39,7 @@ static int Fetch(int sd, char *filename)
 		fd;
 	pi_buffer_t *buffer;
 
-	fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC);
+	fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
 	if (fd < 0)
 		return -1;
 
