@@ -1,5 +1,5 @@
 /*
- * $Id: pilot-xfer.c,v 1.185 2006/10/12 14:21:21 desrod Exp $ 
+ * $Id: pilot-xfer.c,v 1.189 2009-12-11 01:48:34 judd Exp $ 
  *
  * pilot-xfer.c:  Palm Database transfer utility
  *
@@ -709,7 +709,7 @@ pi_file_retrieve_VFS(const int fd, const char *basename, const int socket, const
 	char         rpath[vfsMAXFILENAME];
 	int          rpathlen = vfsMAXFILENAME;
 	FileRef      file;
-	long         attributes;
+	unsigned long attributes;
 	pi_buffer_t  *buffer;
 	ssize_t      readsize,writesize;
 	int          filesize;
@@ -851,7 +851,7 @@ palm_fetch_VFS(const char *dbname, const char *vfspath)
 	fflush(stdout);
 
 	/* Calculate basename, perhaps? */
-	fd = open(dbname,O_WRONLY | O_CREAT | O_TRUNC,0666);
+	fd = open(dbname,O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
 	if (fd < 0) {
 		fprintf(stderr,"\n   Cannot open local file for '%s'.\n",dbname);
 		return;
@@ -1087,7 +1087,7 @@ palm_restore(const char *dirname)
 				break;
 		}
 
-		if (sbuf.st_size > Card.ramFree)
+		if ((unsigned long)sbuf.st_size > Card.ramFree)
 		{
 			fprintf(stderr, "\n\n");
 			fprintf(stderr, "   Insufficient space to install this file on your Palm.\n");
@@ -1193,7 +1193,7 @@ static int pi_file_install_VFS(const int fd, const char *basename, const int soc
 	char        rpath[vfsMAXFILENAME];
 	int         rpathlen = vfsMAXFILENAME;
 	FileRef     file;
-	long        attributes;
+	unsigned long attributes;
 	char        *filebuffer = NULL;
 	long        volume = -1;
 	long        used,
@@ -1228,7 +1228,7 @@ static int pi_file_install_VFS(const int fd, const char *basename, const int soc
 	/* Calculate free space but leave last 64k free on card */
 	freespace  = total - used - 65536 ;
 
-	if (sbuf.st_size > freespace)
+	if ((unsigned long)sbuf.st_size > freespace)
 	{
 		fprintf(stderr, "\n\n");
 		fprintf(stderr, "   Insufficient space to install this file on your Palm.\n");
@@ -1319,6 +1319,14 @@ static int pi_file_install_VFS(const int fd, const char *basename, const int soc
 		return bad_vfs_path;
 	}
 
+	/* If the file already exists we want to truncate it so if we write a smaller file
+	 * the tail of the previous file won't show */
+	if (dlp_VFSFileResize(socket, file, 0) < 0)
+	{
+		fprintf(stderr,"   Cannot truncate file size to 0 '%s'.\n",rpath);
+		/* Non-fatal error, continue */
+	}
+
 #define FBUFSIZ 65536
 	filebuffer = (char *)malloc(FBUFSIZ);
 	if (NULL == filebuffer)
@@ -1367,6 +1375,7 @@ static int pi_file_install_VFS(const int fd, const char *basename, const int soc
 cleanup:
 	free(filebuffer);
 	dlp_VFSFileClose(socket,file);
+   
 	close(fd);
 	return sbuf.st_size;
 }
@@ -1431,7 +1440,7 @@ static void palm_install_internal(const char *filename)
 			break;
 	}
 
-	if (sbuf.st_size > Card.ramFree)
+	if ((unsigned long)sbuf.st_size > Card.ramFree)
 	{
 		fprintf(stderr, "   ERROR: Insufficient space to install this file on your Palm.\n");
 		fprintf(stderr, "          We needed %lu and only had %lu available..\n\n",
